"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault").default;
var _react = require("@testing-library/react");
var _userEvent = _interopRequireDefault(require("@testing-library/user-event"));
var _vitest = require("vitest");
require("@testing-library/jest-dom");
var _runAxeCheck = require("@instructure/ui-axe-check/lib/runAxeCheck.js");
var _index = require("../index");
var _jsxRuntime = require("@emotion/react/jsx-runtime");
var _TextInput, _TextInput2, _TextInput3, _TextInput4, _TextInput5, _TextInput6, _TextInput7;
/*
 * The MIT License (MIT)
 *
 * Copyright (c) 2015 - present Instructure, Inc.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
describe('<TextInput/>', () => {
  let consoleErrorMock;
  beforeEach(() => {
    // Mocking console to prevent test output pollution and expect for messages
    consoleErrorMock = _vitest.vi.spyOn(console, 'error').mockImplementation(() => {});
  });
  afterEach(() => {
    consoleErrorMock.mockRestore();
  });
  it('should include a label', async () => {
    const _render = (0, _react.render)(_TextInput || (_TextInput = (0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name"
      }))),
      container = _render.container;
    const label = container.querySelector('label');
    expect(label).toHaveTextContent('Name');
  });
  it('should focus the input when focus is called', async () => {
    var _ref;
    let ref;
    (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
      renderLabel: "Name"
      //@ts-expect-error TODO this is coming from ReactComponentWrapper
      ,
      inputRef: el => {
        ref = el;
      }
    }));
    const input = _react.screen.getByRole('textbox');
    (_ref = ref) === null || _ref === void 0 ? void 0 : _ref.focus();
    expect(input).toHaveFocus();
  });
  it('should provide an inputRef prop', async () => {
    const inputRef = _vitest.vi.fn();
    (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
      renderLabel: "Name",
      inputRef: inputRef
    }));
    const input = _react.screen.getByRole('textbox');
    expect(inputRef).toHaveBeenCalledWith(input);
  });
  it('should provide a value getter', async () => {
    var _ref2;
    let ref;
    (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
      renderLabel: "Name",
      defaultValue: "bar"
      //@ts-expect-error TODO this is coming from ReactComponentWrapper
      ,
      inputRef: el => {
        ref = el;
      }
    }));
    expect((_ref2 = ref) === null || _ref2 === void 0 ? void 0 : _ref2.value).toBe('bar');
  });
  it('should let aria-describedby through', async () => {
    (0, _react.render)(_TextInput2 || (_TextInput2 = (0, _jsxRuntime.jsx)(_index.TextInput, {
      renderLabel: "Name",
      "aria-describedby": "abcd"
    })));
    const input = _react.screen.getByRole('textbox');
    expect(input).toHaveAttribute('aria-describedby', 'abcd');
  });
  describe('events', () => {
    it('responds to onChange event', async () => {
      const onChange = _vitest.vi.fn();
      (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        onChange: onChange
      }));
      const input = _react.screen.getByRole('textbox');
      _react.fireEvent.change(input, {
        target: {
          value: 'foo'
        }
      });
      await (0, _react.waitFor)(() => {
        expect(onChange).toHaveBeenCalledTimes(1);
      });
    });
    it('responds to onBlur event', async () => {
      const onBlur = _vitest.vi.fn();
      (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        onBlur: onBlur
      }));
      _userEvent.default.tab();
      _userEvent.default.tab();
      await (0, _react.waitFor)(() => {
        expect(onBlur).toHaveBeenCalled();
      });
    });
    it('responds to onFocus event', async () => {
      const onFocus = _vitest.vi.fn();
      (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        onFocus: onFocus
      }));
      const input = _react.screen.getByRole('textbox');
      input.focus();
      await (0, _react.waitFor)(() => {
        expect(onFocus).toHaveBeenCalled();
      });
    });
  });
  describe('interaction', () => {
    it('should set the disabled attribute when `interaction` is disabled', async () => {
      (0, _react.render)(_TextInput3 || (_TextInput3 = (0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        interaction: "disabled"
      })));
      const input = _react.screen.getByRole('textbox');
      expect(input).toHaveAttribute('disabled');
    });
    it('should set the disabled attribute when `disabled` is set', async () => {
      (0, _react.render)(_TextInput4 || (_TextInput4 = (0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        disabled: true
      })));
      const input = _react.screen.getByRole('textbox');
      expect(input).toHaveAttribute('disabled');
    });
    it('should set the readonly attribute when `interaction` is readonly', async () => {
      (0, _react.render)(_TextInput5 || (_TextInput5 = (0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        interaction: "readonly"
      })));
      const input = _react.screen.getByRole('textbox');
      expect(input).toHaveAttribute('readonly');
    });
    it('should set the readonly attribute when `readOnly` is set', async () => {
      (0, _react.render)(_TextInput6 || (_TextInput6 = (0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        readOnly: true
      })));
      const input = _react.screen.getByRole('textbox');
      expect(input).toHaveAttribute('readonly');
    });
  });
  describe('for a11y', () => {
    it('should meet standards', async () => {
      const _render2 = (0, _react.render)(_TextInput7 || (_TextInput7 = (0, _jsxRuntime.jsx)(_index.TextInput, {
          renderLabel: "Name"
        }))),
        container = _render2.container;
      const axeCheck = await (0, _runAxeCheck.runAxeCheck)(container);
      expect(axeCheck).toBe(true);
    });
    it('should set aria-invalid when errors prop is set', async () => {
      (0, _react.render)((0, _jsxRuntime.jsx)(_index.TextInput, {
        renderLabel: "Name",
        messages: [{
          type: 'error',
          text: 'some error message'
        }]
      }));
      const input = _react.screen.getByRole('textbox');
      expect(input).toHaveAttribute('aria-invalid');
    });
  });
});